'''
Created on Jul 11, 2009
Modified on Oct 8, 2009

@author: yongjin
'''
import sys; import os;
sys.path.append( os.path.abspath('.') )

import networkx as nx
from random import random, randint
from numpy import zeros, dot, diag, sum, argwhere
from scipy.io import savemat

## generate partial time varying data
def seqtimedata(NN=20, TT=10, KK=3, Pwithin = .7, Pbetween = .1, Pswitch = .1):
    '''
    dataGraphs, timeGraph, vertices, Ztrue = 
    seqtimedata(NN=20, TT=10, KK=3, Pwithin = .7, Pbetween = .1, Pswitch = .1)
    '''
    Z = zeros((NN,KK)); 
    for ii in xrange(NN): Z[ii,randint(0,KK-1)] = 1.;    
    
    vertices = [str(ii) for ii in xrange(NN)]
    Ztrue = []; dataGraphs = []; timeGraph = nx.Graph(); timeGraph.add_node(0)
    
    for tt in xrange(TT):
        
        # 1. switch membership
        for ii in xrange(NN):
            if random() <= Pswitch : 
                Z[ii,:] = 0.;
                Z[ii,randint(0,KK-1)] = 1.;
                pass
            pass
        
        Ztrue.append( Z.copy() )
        # 2. sample edges
        edges = []
        # within edge        
        for kk in xrange(KK):
            edges.extend([ (str(ii[0]),str(jj[0])) for ii in argwhere(Z[:,kk])
                           for jj in argwhere(Z[:,kk]) if random() <= Pwithin ])
                
        # between edge
        edges.extend([ (str(ii[0]),str(jj[0])) for kk in xrange(KK)
                       for ii in argwhere( Z[:,kk] )
                       for jj in argwhere( Z[:,(kk+1)%KK] ) 
                       if random() <= Pbetween ])
                            
        gtt = nx.Graph(); gtt.add_edges_from( edges );
        dataGraphs.append( gtt )
        if tt > 0: timeGraph.add_edge(tt,(tt-1))
        pass
    
    return dataGraphs, timeGraph, vertices, Ztrue

############

if __name__ == '__main__' : 
    
    # data generation parameters
    NN=20         # number of vertices
    TT=10         # number of time slots
    KK=3          # number of clusters
    Pwithin=.6    # Pr{ within cluster edge }
    Pbetween=.2   # Pr{ between cluster edge }
    Pswitch=.1    # Pr{ membership switch }
    
    # generate time-varying network data 
    dataGs, timeG, vertices, Ztrue = seqtimedata(NN,TT,KK,Pwithin,Pbetween,Pswitch)
    
    # save data graphs
    for ii, dg in enumerate(dataGs) : 
        nx.write_adjlist( dg, '../data/G%d.adjlist.gz'%ii )
        
    # save time-dependency graph
    nx.write_adjlist( timeG, '../data/T.adjlist.gz' )
    
    # write out vertex names (not to confuse order of names)
    fh = open( '../data/vertices.txt', 'w' )
    fh.writelines( ['%s\n'%v for v in vertices] )
    fh.close()
    
    # write out true labeling
    Zout = {}
    for ii, z in enumerate( Ztrue ) :
        Zout['true%d'%ii] = z
    savemat("../data/true.mat",Zout,appendmat=False)
    
    pass


